package extension

import (
	"encoding/json"
	"fmt"
	"io"
	"net/http"

	"codeberg.org/thefederationinfo/nodeinfo-go"
	known_extension "codeberg.org/thefederationinfo/nodeinfo_metadata_survey"
)

func init() {
	nodeinfo.RegisterExtension(&mastodon{})
}

type mastodon struct{}

func (mastodon) Name() string {
	return "mastodon"
}

type mastodonInstance struct {
	Title       string `json:"title"`
	Description string `json:"description"`
	Contact     struct {
		Email   string `json:"email"`
		Account struct {
			Username    string `json:"username"`
			DisplayName string `json:"display_name"`
		} `json:"account"`
	} `json:"contact"`
}

func (mastodon) Extend(c *http.Client, domain string, n *nodeinfo.Node) error {
	if n.Metadata == nil {
		n.Metadata = make(map[string]interface{})
	}

	if n.Instance.Name == "" {
		n.Instance.Name, _ = n.Metadata[known_extension.KeyName].(string)
		delete(n.Metadata, known_extension.KeyName)
	}
	if n.Instance.Description == "" {
		n.Instance.Description, _ = n.Metadata[known_extension.KeyDescription].(string)
		delete(n.Metadata, known_extension.KeyDescription)
	}
	_, adminsOK := n.Metadata[known_extension.KeyAdmins]

	if n.Instance.Name == "" || n.Instance.Description == "" || !adminsOK {
		n.Version = "2.2"
		resp, err := c.Get(fmt.Sprintf("https://%s/api/v2/instance", domain))
		if err != nil {
			return fmt.Errorf("could not query mastodon specific instance endpoint: %w", err)
		}
		rawResp, err := io.ReadAll(resp.Body)
		if err != nil {
			return err
		}

		mi := &mastodonInstance{}
		if err := json.Unmarshal(rawResp, mi); err != nil {
			return err
		}

		if n.Instance.Name == "" {
			n.Instance.Name = mi.Title
		}
		if n.Instance.Description == "" {
			n.Instance.Description = mi.Description
		}
		if !adminsOK {
			n.Metadata[known_extension.KeyAdmins] = []known_extension.NodeAdmin{{
				Name:    mi.Contact.Account.DisplayName,
				Account: mi.Contact.Account.Username,
				Email:   mi.Contact.Email,
			}}
		}
	}

	return nil
}
